/*
 *
 *   dump.cc
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 */


#include <fstream>
#include <iostream>
#include <string>
#include <list>
#include <my++/muxunixsocket.h>
#include <my++/muxtty.h>
#include <my++/multiplex.h>

#include <my++/termios.h>
#include <my++/serialios.h>
#include <my++/conios.h>

#include <cstring>
#include <cstdio>

#include "util.h"

using namespace std;

const int default_flags = my::device::not_ctl_tty | my::device::nonblocking;


std::string dump (const char *buffer, size_t count) {
	std::string result;

	for (size_t i = 0; i < count; ++i) {
		result += "[" + itoa(buffer[i]);
		if (isalnum(buffer[i]) || ispunct(buffer[i]) || (buffer[i] == ' ')) {
			result += ':';
			result += buffer[i];
		}
		result += "]";
		if (buffer[i] == '\n')
			result += "\r\n";
	}
	return result;
}

/*
 *
 *   main (...)
 *
 */

int main (int argc, char **argv) {
	if (argc < 3) {
		cerr << "Usage: dump <socket_path> <dump_file>" << endl;
		return 0;
	}

	my::multiplexed::unixsocket sock;
	my::multiplexed::tty console;
	my::multiplex mux;
	std::string remote_address(argv[1]);
	std::string filename(argv[2]);
	std::string message;
	char buffer [128];
	int count, rtn;

	my::termios console_config;

	ofstream outfile (filename.c_str());
	if (!outfile) {
		cerr << "Could not open \'" << filename << "\' for writing." << endl;
		return 0;
	}

	try {
		console.open("/dev/tty", default_flags);
		console.set_options (console_config);
	} catch (const std::exception &e) {
		cerr << "Error opening /dev/tty: " << e.what() << endl;
		return 1;
	}

	for (;;) {
		cerr << "Try to connect to socket \'" << remote_address << "\': ";
		cerr.flush();

		try {
			sock.connect(remote_address);
		} catch (const std::exception &e) {
			cerr << "Error opening socket: " << e.what() << endl;
			return 1;
		}

		cerr << "done!\r\n\n";

		/* */
		mux.add(sock);
		mux.add(console);

		for (;;) {
			rtn = mux.watch();

			if (rtn > 0) {
				if (console.closed()) {
					console.close();
					sock.close();
					return 0;
				}

				if (sock.disconnected()) {
					console.close();
					sock.close();
					return 0;
				}

				while (console.ready()) {
					count = console.get(buffer, sizeof(buffer));
					//cerr << "Read " << count << " bytes from CONSOLE.\r\n";
					if (buffer[0] == 3)
						return 0;
					if (count > 0)
						sock.put(buffer, count);
				}

				while (sock.ready()) {
					count = sock.get(buffer, sizeof(buffer));
					if (count > 0) {
						console.put(buffer, count);
						// write to the dump file
						message = dump(buffer, count);
						outfile << message;
						outfile.flush();
					}
				}
			}
		}

		mux.remove(sock);
	}

	/* should never reach here */
	return 0;
}
