/*
 *
 *
 *   filter.h
 *
 *
 */


#ifndef __FILTER_H
	#define __FILTER_H

	class Filter {
		public:
			virtual std::string filter(const std::string &s) { return filter(s.c_str(), s.size()); };
			virtual std::string filter(const char *buf, size_t len) = 0;
			virtual ~Filter (void) { /* noop */ };
	};


	class RISFilter : public Filter {
		private:
			bool escape, cr, lf;

		public:
			RISFilter(void) : escape(false), cr(false), lf(false) { /* noop */ };
			~RISFilter(void) { /* noop */ };

		public:
			std::string filter(const char *buf, size_t len);
	};


	class SGFilter : public Filter {
		private:
			bool cr, lf;

		public:
			SGFilter(void) : cr(false), lf(false) { /* noop */ };
			~SGFilter(void) { /* noop */ };

		public:
			std::string filter(const char *buf, size_t len);
	};


	inline std::string RISFilter::filter(const char *buf, size_t len) {
		const char *cp = 0;
		char ch;
		std::string result;

		for (cp = buf; cp != buf + len; ++cp) {
			ch = *cp;

			if (escape) {
				escape = false;
				cr = false;
				lf = false;
			} else if (ch == '\r') {
				if (! lf) {
					result += '\n';
					cr = true;
				} else {
					cr = false;
				}
				cr = true;
				lf = false;
			} else if (ch == '\n') {
				if (! cr) {
					result += '\n';
					lf = true;
				} else {
					lf = false;
				}
				cr = false;
			} else if (ch == '\x1B') {
				escape = true;
				cr = false;
				lf = false;
			} else if (isprint(ch)) {
				result += ch;
				escape = false;
				cr = false;
				lf = false;
			}
		}

		return result;
	}


	inline std::string SGFilter::filter(const char *buf, size_t len) {
		const char *cp = 0;
		std::string result;
		char ch;

		for (cp = buf; cp != buf + len; ++cp) {
			ch = *cp;
			if (ch == '\r') {
				if (! lf) {
					result += '\n';
					cr = true;
				} else {
					cr = false;
				}
				cr = true;
				lf = false;
			} else if (ch == '\n') {
				if (! cr) {
					result += '\n';
					lf = true;
				} else {
					lf = false;
				}
				cr = false;
			} else if (isprint(ch)) {
				result += ch;
				cr = false;
				lf = false;
			}
		}

		return result;
	}

#endif /* __FILTER_H */
