/*
 *
 *
 *   rtwin.cc
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 *
 */


#include <iostream>
#include <fstream>
#include <string>

#include <cctype>
#include <curses.h>

#include "rtwin.h"


// allow empty lines
const static bool empty = true;

// work in std:: namespace
using namespace std;


/*
 *
 *   RISTermWindow::operator=(...)
 *
 */

RISTerminalWindow &RISTerminalWindow::operator = (WINDOW *w) {
	if (w) {
		win = w;
		wcolor_set(win, rf_normal, NULL);
	}
	return(*this);
}


/*
 *
 *   RISTermWindow::init()
 *
 */

void RISTerminalWindow::init(void) {
	/*
	init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
	init_pair(COLOR_RED,   COLOR_RED,   COLOR_BLACK);
	init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
	*/
}


/*
 *
 *   RISTermWindow::RISTermWindow(...)
 *
 */

RISTerminalWindow::RISTerminalWindow(WINDOW *w, short normal, short alarm)
	: win(w), rf_alarm(alarm), rf_normal(normal), escape(false), cr(false) {
	init();
	wcolor_set(win, rf_normal, NULL);
}


/*
 *
 *   RISTermWindow::RISTermWindow(...)
 *
 */

RISTerminalWindow::RISTerminalWindow(short normal, short alarm)
	: win(NULL), rf_alarm(alarm), rf_normal(normal), escape(false), cr(false) {
	init();
}


/*
 *
 *   RISTermWindow::write(...)
 *
 */

int RISTerminalWindow::write (const unsigned char *buffer, size_t count) {
	const unsigned char *cp = 0;

	if (count == 0)
		return 0;

	for (cp = buffer; cp != (buffer + count); ++cp) {
		switch (*cp) {
			//case '\r':
			case '\n':
				if (escape)
					break;
				if (!cr) {
					wattroff(win, A_BOLD);
					wcolor_set(win, rf_normal, NULL);
					if (waddch(win, '\n') == ERR)
						return ERR;
				}
				cr = true;
				escape = false;
				break;

			case '\x1B':
				escape = true;
				cr = false;
				break;

			default:
				if (escape) {
					if ((*cp == '3') && (rf_alarm >= 0)) {
						wattron(win, A_BOLD);
						wcolor_set(win, rf_alarm, NULL);
					} else if ((*cp == '4') && (rf_normal >= 0)) {
						wattroff(win, A_BOLD);
						wcolor_set(win, rf_normal, NULL);
					}
				} else {
					if (isprint(*cp)) {
						if (waddch(win, (*cp)) == ERR)
							return ERR;
					}
				}

				if (empty)
					cr = false;
				escape = false;
				break;
		}
	}

	// output the result
	return OK;
}


void RISTerminalWindow::test (void) {
	if (!win)
		return;

	std::string test_string;

	test_string  = "\x1B""4This is a test.\r\n";
	test_string += "\x1B""3This is a test.\r\n";
	test_string += "\x1B""4This is a test.\r\n";
	test_string += "\x1B""3This is a test.\r\n";
	test_string += "\x1B""4This is a test.\r\n";
	test_string += "\x1B""3This is a test.\r\n";

	write(test_string);
}


int RISTerminalWindow::put (int ch) {
	int result = waddch(win, ch);
	if (result == OK)
		wrefresh(win);
	return result;
}

// EOF
