/*
 *
 *
 *    sgdserver.h
 *
 *    Simple C++ application class to gateway a serial device into a Unix
 *    socket, to allow multiple clients to connect simultaneously.
 *
 *    Copyright (C) 2003 by Matt Roberts,
 *    All rights reserved.
 *
 *
 */


#ifndef __SGSERVER_H
#define __SGSERVER_H

	// my++ headers
	#include <my++/unixsocket.h>
	#include <my++/tty.h>
	#include <my++/serialios.h>
	#include <my++/auth.h>
	#include <my++/daemon.h>
	#include <my++/select.h>

	// system headers
	#include <syslog.h>
	#include <stdlib.h>
	#include <sys/types.h>
	#include <sys/stat.h>
	#include <signal.h>

	// C++ headers
	#include <string>
	#include <iostream>
	#include <list>
	#include <algorithm>

	// local headers
	#include "sgdconfig.h"

	/*
	 *
	 *   class server
	 *
	 */
	
	class server {
		private:
			// an I/O buffer type
			typedef std::deque<unsigned char> iobuffer;
	
			// a client structure
			struct client {
				my::unixsocket sock_obj;       // a *nix-domain socket for talking to client
				iobuffer tx_buffer, rx_buffer; // I/O buffers
				bool dead;                     // a flag to say we are done with this object
				client(void) : dead(false) { /* noop */ }; // ctor
			};

			// a client list type
			typedef std::list<client *> clist;

			// a port structure
			struct port {
				port_config config;      // config object
				my::unixsocket listener; // *nix-domain listen socket
				my::tty device;          // TTY device
				iobuffer tx_buffer, rx_buffer; // I/O buffers
				clist clients;           // list of clients
			};
	
			// a client list type
			typedef std::list<port *> plist;
	
		private:
			plist ports;            // list of ports for this server
	
			bool s_daemon;          // config: daemonize on startup
			bool s_syslog;          // config: log runtime events to syslog(3)
			bool s_stderr;          // config: log runtime events to stderr
			std::string s_pidfile;  // path to PID file

		protected:
			// utility functions
			void dolog (const std::string &message, bool alarm = false);
			void dolog (/*const*/ my::serialios &s);
			void iosetup (my::fdset &rset, my::fdset &wset);

			client *find (const my::unixsocket *who);
			size_t count (const port *who = 0);

		public:
			server(const server_config &cfg);  // ctor
			~server(void) throw();             // dtor

			int run (void);  // main server code

	};
	
#endif /* __SGSERVER_H */
